--[[
Baggins_ItemRack

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Thanks to Nargiddley for writing Baggins and Gello for writing ItemRack

License

Copyright (c) 2007 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


TODO:
Localize
Consider some sort of table compositing... should not be necassary though sets only time
tables are created/removed enmase is when a new set is added, or an old set is removed.

]]

Baggins_ItemRack = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceHook-2.1")
local dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
local sieve
local item_rack_user
local other_users_scanned
local items_table = {}
local sets_table = {}
local user_table = {}

local function link_to_item_rack_id(link)
   if link then
      local _,_,id = link:find("item:([%-%d]+:[%-%d]+:[%-%d]+):[%-%d]+")
      return id
   end
   return nil
end

-- We hook the following
-- ItemRack_Sets_Initialize -> Sets initialized
-- ItemRack_Sets_Saved_OnClick -> New set defined
-- ItemRack_Sets_Remove_OnClick -> Set removed
function Baggins_ItemRack:CacheUser(u)
   local i,setname,set

   --self:Print("Caching: "..u)
   local _,_,user,realm = u:find("(.*) of (.*)")
   if not user or not realm then
      -- Unexpected format for ItemRack user
      user = u
      realm = "Other"
   end
   if not user_table[realm] then
      user_table[realm] = {}
   end
   user_table[realm][user] = u

   -- Remove any items mappings for this user
   local item_table = {}
   items_table[u] = item_table
   local set_table = {}
   sets_table[u] = set_table

   local usersets = Rack and Rack_User and Rack_User[u] and Rack_User[u].Sets
   if not usersets then return end

   local other = (u ~= item_rack_user)
   local other_item_table
   other_item_table = items_table.Other
   if not other_item_table then
      other_item_table = {}
      items_table.Other = other_item_table
   end
   local other_set_table
   other_set_table = sets_table.Other
   if not other_set_table then
      other_set_table = {}
      sets_table.Other = other_set_table
   end

   for setname, set in pairs(usersets) do
      if not setname:find("^ItemRack-") and not setname:find("^Rack-") then
	 set_table[setname] = true
	 if other then
	    other_set_table[setname] = true
	 end
	 for i=0,19 do
	    local setitem = set[i] and set[i].id
	    if setitem then
	       if not item_table[setitem] then item_table[setitem] = {} end
	       --[[if not item_table[setitem][setname] then
		  self:Print("Adding: "..setitem.." to set "..setname.." for "..u)
	       end]]
	       item_table[setitem][setname] = true
	       if other then
		  if not other_item_table[setitem] then other_item_table[setitem] = {} end
		  --[[if not other_item_table[setitem][setname] then
		     self:Print("Adding: "..setitem.." to set "..setname.." - Others")
		  end]]
		  other_item_table[setitem][setname] = true
	       end
	    end
	 end
      end
   end
end

function Baggins_ItemRack:SetsUpdated()
   local sets,setname,ids,user

   if not Rack then return end
   if not Rack_User then return end
   if not item_rack_user then return end
   --self:Print("Updating ItemRack sets")

   if other_users_scanned then
      self:CacheUser(item_rack_user)
   else
      for user in pairs(Rack_User) do
	 self:CacheUser(user)
      end
      other_users_scanned = true
   end

   -- Fix up set table for "Any" user
   local any = {}
   sets_table.Any = any
   for sets in pairs(sets_table[item_rack_user]) do
      any[sets] = true
   end
   for sets in pairs(sets_table.Other) do
      any[sets] = true
   end

   -- Fix up item table for "Any" user
   any = {}
   items_table.Any = any
   for ids,sets in pairs(items_table[item_rack_user]) do
      if not any[ids] then any[ids] = {} end
      for setname in pairs(sets) do
	 any[ids][setname] = true
      end
   end
   for ids,sets in pairs(items_table.Other) do
      if not any[ids] then any[ids] = {} end
      for setname in pairs(sets) do
	 any[ids][setname] = true
      end
   end
   -- Cause Baggins to update all the bags just in case.
   if sieve then sieve:ForceFullUpdate() end
end

function Baggins_ItemRack:ItemRack_Sets_Initialize(...)
   self.hooks.ItemRack_Sets_Initialize(...)
   self:SetsUpdated()
end

function Baggins_ItemRack:ItemRack_Sets_Save_OnClick(...)
   self.hooks.ItemRack_Sets_Save_OnClick(...)
   self:SetsUpdated()
end

function Baggins_ItemRack:ItemRack_Sets_Remove_OnClick(...)
   self.hooks.ItemRack_Sets_Remove_OnClick(...)
   self:SetsUpdated()
end

function Baggins_ItemRack.Matches(bag,slot,rule)
   local self = Baggins_ItemRack
   local id = link_to_item_rack_id(GetContainerItemLink(bag, slot))
   if not id then return end

   local set = rule.set
   if not set then return end
   local user = rule.user
   if not user then return end
   if user == "Self" then user = item_rack_user end
   if not user then return end

   local invert = nil
   if set == "None" then
      set = "Any"
      invert = true
   end

   local res = items_table[user]
   res = res and res[id]
   if set ~= "Any" then res = res and res[set] end
   if invert then res = not res end
   return res
end

function Baggins_ItemRack.GetName(rule)
   local self = Baggins_ItemRack
   return "ItemRack User:"..(rule.user or "").." Set:"..(rule.set or "")
end

function Baggins_ItemRack.CleanRule(rule)
   rule.user = "Self"
   rule.set = "Any"
end

function Baggins_ItemRack.DewDropOptions(rule, level, value)
   local self = Baggins_ItemRack
   if level == 1 then
      dewdrop:AddLine('text', "Item Rack Sets", 'isTitle', true)
      dewdrop:AddLine('text', "User Name", 'hasArrow', true,'value',"ItemRackUser")
      dewdrop:AddLine('text', "Set Name", 'hasArrow', true, 'value', "ItemRackSet")
   elseif level == 2 and value == "ItemRackUser" then
      dewdrop:AddLine('text', "Any", "checked", rule.user == "Any","func",function(k) rule.user = "Any" rule.set = "Any" sieve:OnRuleChanged() end,"arg1","Any")
      dewdrop:AddLine('text', "Self", "checked", rule.user == "Self","func",function(k) rule.user = "Self" rule.set = "Any" sieve:OnRuleChanged() end,"arg1","Self")
      local realm
      for realm in pairs(user_table) do
	 dewdrop:AddLine('text', "Realm: "..realm, 'hasArrow', true, 'value', 'Realm:'..realm)
      end
   elseif level == 3 then
      local _,_,realm = value:find("Realm:(.*)")
      if realm then
	 local user
	 local full
	 for user,full in pairs(user_table[realm]) do
	    dewdrop:AddLine('text', user, "checked", rule.user == full,"func",function(k) rule.user = full rule.set = "Any" sieve:OnRuleChanged() end,"arg1",full)
	 end
      end
   elseif level == 2 and value == "ItemRackSet" then
      dewdrop:AddLine('text', "Any", "checked", rule.set == "Any","func",function(k) rule.set = "Any" sieve:OnRuleChanged() end,"arg1","Any")
      dewdrop:AddLine('text', "None", "checked", rule.set == "None","func",function(k) rule.set = "None" sieve:OnRuleChanged() end,"arg1","None")
      local user = rule.user
      -- Treat self as "Any" here.. so that we include sets from other users...
      -- since this is probably a shared category.
      if user and user == "Self" then user = "Any" end
      local usersets = sets_table[user]
      if usersets then
	 local setnames = {}
	 for set in pairs(usersets) do
	    table.insert(setnames,set)
	 end
	 table.sort(setnames)
	 for _,set in ipairs(setnames) do
	    dewdrop:AddLine('text', set, "checked", rule.set == set,"func",function(k) rule.set = set sieve:OnRuleChanged() end,"arg1",set)
	 end
      end
   end
end

function Baggins_ItemRack:OnEnable()
   sieve = AceLibrary("Sieve-0.1")

   item_rack_user = UnitName("player").." of "..GetRealmName()

   self:SetsUpdated()
   self:Hook("ItemRack_Sets_Initialize")
   self:Hook("ItemRack_Sets_Save_OnClick")
   self:Hook("ItemRack_Sets_Remove_OnClick")

   sieve:AddCustomRule("ItemRack",{
			  DisplayName = "ItemRack",
			  Description = "Filter by ItemRack set",
			  Matches = self.Matches,
			  GetName = self.GetName,
			  CleanRule = self.CleanRule,
			  DewDropOptions = self.DewDropOptions})
end
